@@ -990,21 +990,32 @@ get_connector_type_name (GfConnectorType connector_type)
 static gboolean
 is_main_tiled_monitor_output (GfOutput *output)
 {
-  return output->tile_info.loc_h_tile == 0 && output->tile_info.loc_v_tile == 0;
+  const GfOutputInfo *output_info;
+
+  output_info = gf_output_get_info (output);
+
+  return (output_info->tile_info.loc_h_tile == 0 &&
+          output_info->tile_info.loc_v_tile == 0);
 }
 
 static void
 rebuild_monitors (GfMonitorManager *manager)
 {
+  GfMonitorManagerPrivate *priv;
+  GList *gpus;
   GList *l;
 
+  priv = gf_monitor_manager_get_instance_private (manager);
+
   if (manager->monitors)
     {
       g_list_free_full (manager->monitors, g_object_unref);
       manager->monitors = NULL;
     }
 
-  for (l = manager->gpus; l; l = l->next)
+  gpus = gf_backend_get_gpus (priv->backend);
+
+  for (l = gpus; l; l = l->next)
     {
       GfGpu *gpu = l->data;
       GList *k;
@@ -1012,14 +1023,17 @@ rebuild_monitors (GfMonitorManager *manager)
       for (k = gf_gpu_get_outputs (gpu); k; k = k->next)
         {
           GfOutput *output = k->data;
+          const GfOutputInfo *output_info;
 
-          if (output->tile_info.group_id)
+          output_info = gf_output_get_info (output);
+
+          if (output_info->tile_info.group_id)
             {
               if (is_main_tiled_monitor_output (output))
                 {
                   GfMonitorTiled *monitor_tiled;
 
-                  monitor_tiled = gf_monitor_tiled_new (gpu, output);
+                  monitor_tiled = gf_monitor_tiled_new (gpu, manager, output);
                   manager->monitors = g_list_append (manager->monitors, monitor_tiled);
                 }
             }
@@ -1038,10 +1052,16 @@ static GList *
 combine_gpu_lists (GfMonitorManager *manager,
                    GList            * (*list_getter) (GfGpu *gpu))
 {
+  GfMonitorManagerPrivate *priv;
+  GList *gpus;
   GList *list = NULL;
   GList *l;
 
-  for (l = manager->gpus; l; l = l->next)
+  priv = gf_monitor_manager_get_instance_private (manager);
+
+  gpus = gf_backend_get_gpus (priv->backend);
+
+  for (l = gpus; l; l = l->next)
     {
       GfGpu *gpu = l->data;
 
@@ -1082,14 +1102,16 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
     {
       GfCrtc *crtc = l->data;
       GVariantBuilder transforms;
-      GfCrtcConfig *crtc_config;
+      const GfCrtcConfig *crtc_config;
 
       g_variant_builder_init (&transforms, G_VARIANT_TYPE ("au"));
       for (j = 0; j <= GF_MONITOR_TRANSFORM_FLIPPED_270; j++)
-        if (crtc->all_transforms & (1 << j))
-          g_variant_builder_add (&transforms, "u", j);
+        {
+          if (gf_crtc_get_all_transforms (crtc) & (1 << j))
+            g_variant_builder_add (&transforms, "u", j);
+        }
 
-      crtc_config = crtc->config;
+      crtc_config = gf_crtc_get_config (crtc);
 
       if (crtc_config != NULL)
         {
@@ -1099,7 +1121,7 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
 
           g_variant_builder_add (&crtc_builder, "(uxiiiiiuaua{sv})",
                                  i, /* ID */
-                                 (int64_t) crtc->crtc_id,
+                                 (int64_t) gf_crtc_get_id (crtc),
                                  crtc_config->layout.x,
                                  crtc_config->layout.y,
                                  crtc_config->layout.width,
@@ -1113,7 +1135,7 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
         {
           g_variant_builder_add (&crtc_builder, "(uxiiiiiuaua{sv})",
                                  i, /* ID */
-                                 (int64_t) crtc->crtc_id,
+                                 (int64_t) gf_crtc_get_id (crtc),
                                  0,
                                  0,
                                  0,
@@ -1127,73 +1149,94 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
 
   for (l = combined_outputs, i = 0; l; l = l->next, i++)
     {
-      GfOutput *output = l->data;
+      GfOutput *output;
+      const GfOutputInfo *output_info;
       GVariantBuilder crtcs, modes, clones, properties;
       GBytes *edid;
       GfCrtc *crtc;
-      gint crtc_index;
+      int crtc_index;
+      int backlight;
+      int min_backlight_step;
+      gboolean is_primary;
+      gboolean is_presentation;
+      const char *connector_type_name;
+      gboolean is_underscanning;
+      gboolean supports_underscanning;
+
+      output = l->data;
+      output_info = gf_output_get_info (output);
 
       g_variant_builder_init (&crtcs, G_VARIANT_TYPE ("au"));
-      for (j = 0; j < output->n_possible_crtcs; j++)
+      for (j = 0; j < output_info->n_possible_crtcs; j++)
         {
           GfCrtc *possible_crtc;
           guint possible_crtc_index;
 
-          possible_crtc = output->possible_crtcs[j];
+          possible_crtc = output_info->possible_crtcs[j];
           possible_crtc_index = g_list_index (combined_crtcs, possible_crtc);
 
           g_variant_builder_add (&crtcs, "u", possible_crtc_index);
         }
 
       g_variant_builder_init (&modes, G_VARIANT_TYPE ("au"));
-      for (j = 0; j < output->n_modes; j++)
+      for (j = 0; j < output_info->n_modes; j++)
         {
           guint mode_index;
 
-          mode_index = g_list_index (combined_modes, output->modes[j]);
+          mode_index = g_list_index (combined_modes, output_info->modes[j]);
           g_variant_builder_add (&modes, "u", mode_index);
 
         }
 
       g_variant_builder_init (&clones, G_VARIANT_TYPE ("au"));
-      for (j = 0; j < output->n_possible_clones; j++)
+      for (j = 0; j < output_info->n_possible_clones; j++)
         {
           guint possible_clone_index;
 
           possible_clone_index = g_list_index (combined_outputs,
-                                               output->possible_clones[j]);
+                                               output_info->possible_clones[j]);
 
           g_variant_builder_add (&clones, "u", possible_clone_index);
         }
 
+      backlight = gf_output_get_backlight (output);
+      min_backlight_step =
+        output_info->backlight_max - output_info->backlight_min
+        ? 100 / (output_info->backlight_max - output_info->backlight_min)
+        : -1;
+      is_primary = gf_output_is_primary (output);
+      is_presentation = gf_output_is_presentation (output);
+      is_underscanning = gf_output_is_underscanning (output);
+      connector_type_name = get_connector_type_name (output_info->connector_type);
+      supports_underscanning = output_info->supports_underscanning;
+
       g_variant_builder_init (&properties, G_VARIANT_TYPE ("a{sv}"));
       g_variant_builder_add (&properties, "{sv}", "vendor",
-                             g_variant_new_string (output->vendor));
+                             g_variant_new_string (output_info->vendor));
       g_variant_builder_add (&properties, "{sv}", "product",
-                             g_variant_new_string (output->product));
+                             g_variant_new_string (output_info->product));
       g_variant_builder_add (&properties, "{sv}", "serial",
-                             g_variant_new_string (output->serial));
+                             g_variant_new_string (output_info->serial));
       g_variant_builder_add (&properties, "{sv}", "width-mm",
-                             g_variant_new_int32 (output->width_mm));
+                             g_variant_new_int32 (output_info->width_mm));
       g_variant_builder_add (&properties, "{sv}", "height-mm",
-                             g_variant_new_int32 (output->height_mm));
+                             g_variant_new_int32 (output_info->height_mm));
       g_variant_builder_add (&properties, "{sv}", "display-name",
-                             g_variant_new_string (output->name));
+                             g_variant_new_string (output_info->name));
       g_variant_builder_add (&properties, "{sv}", "backlight",
-                             g_variant_new_int32 (output->backlight));
+                             g_variant_new_int32 (backlight));
       g_variant_builder_add (&properties, "{sv}", "min-backlight-step",
-                             g_variant_new_int32 ((output->backlight_max - output->backlight_min) ?
-                                                  100 / (output->backlight_max - output->backlight_min) : -1));
+                             g_variant_new_int32 (min_backlight_step));
       g_variant_builder_add (&properties, "{sv}", "primary",
-                             g_variant_new_boolean (output->is_primary));
+                             g_variant_new_boolean (is_primary));
       g_variant_builder_add (&properties, "{sv}", "presentation",
-                             g_variant_new_boolean (output->is_presentation));
+                             g_variant_new_boolean (is_presentation));
       g_variant_builder_add (&properties, "{sv}", "connector-type",
-                             g_variant_new_string (get_connector_type_name (output->connector_type)));
+                             g_variant_new_string (connector_type_name));
       g_variant_builder_add (&properties, "{sv}", "underscanning",
-                             g_variant_new_boolean (output->is_underscanning));
+                             g_variant_new_boolean (is_underscanning));
       g_variant_builder_add (&properties, "{sv}", "supports-underscanning",
-                             g_variant_new_boolean (output->supports_underscanning));
+                             g_variant_new_boolean (supports_underscanning));
 
       edid = manager_class->read_edid (manager, output);
 
@@ -1205,18 +1248,21 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
           g_bytes_unref (edid);
         }
 
-      if (output->tile_info.group_id)
+      if (output_info->tile_info.group_id)
         {
-          g_variant_builder_add (&properties, "{sv}", "tile",
-                                 g_variant_new ("(uuuuuuuu)",
-                                                output->tile_info.group_id,
-                                                output->tile_info.flags,
-                                                output->tile_info.max_h_tiles,
-                                                output->tile_info.max_v_tiles,
-                                                output->tile_info.loc_h_tile,
-                                                output->tile_info.loc_v_tile,
-                                                output->tile_info.tile_w,
-                                                output->tile_info.tile_h));
+          GVariant *tile_variant;
+
+          tile_variant = g_variant_new ("(uuuuuuuu)",
+                                        output_info->tile_info.group_id,
+                                        output_info->tile_info.flags,
+                                        output_info->tile_info.max_h_tiles,
+                                        output_info->tile_info.max_v_tiles,
+                                        output_info->tile_info.loc_h_tile,
+                                        output_info->tile_info.loc_v_tile,
+                                        output_info->tile_info.tile_w,
+                                        output_info->tile_info.tile_h);
+
+          g_variant_builder_add (&properties, "{sv}", "tile", tile_variant);
         }
 
       crtc = gf_output_get_assigned_crtc (output);
@@ -1224,10 +1270,10 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
 
       g_variant_builder_add (&output_builder, "(uxiausauaua{sv})",
                              i, /* ID */
-                             (gint64) output->winsys_id,
+                             gf_output_get_id (output),
                              crtc_index,
                              &crtcs,
-                             output->name,
+                             gf_output_get_name (output),
                              &modes,
                              &clones,
                              &properties);
@@ -1236,14 +1282,17 @@ gf_monitor_manager_handle_get_resources (GfDBusDisplayConfig   *skeleton,
   for (l = combined_modes, i = 0; l; l = l->next, i++)
     {
       GfCrtcMode *mode = l->data;
+      const GfCrtcModeInfo *crtc_mode_info;
+
+      crtc_mode_info = gf_crtc_mode_get_info (mode);
 
       g_variant_builder_add (&mode_builder, "(uxuudu)",
                              i, /* ID */
-                             (gint64) mode->mode_id,
-                             (guint32) mode->width,
-                             (guint32) mode->height,
-                             (gdouble) mode->refresh_rate,
-                             (guint32) mode->flags);
+                             (int64_t) gf_crtc_mode_get_id (mode),
+                             (uint32_t) crtc_mode_info->width,
+                             (uint32_t) crtc_mode_info->height,
+                             (double) crtc_mode_info->refresh_rate,
+                             (uint32_t) crtc_mode_info->flags);
     }
 
   if (!gf_monitor_manager_get_max_screen_size (manager,
@@ -1279,6 +1328,8 @@ gf_monitor_manager_handle_change_backlight (GfDBusDisplayConfig   *skeleton,
   GfMonitorManagerClass *manager_class;
   GList *combined_outputs;
   GfOutput *output;
+  const GfOutputInfo *output_info;
+  int new_backlight;
 
   manager_class = GF_MONITOR_MANAGER_GET_CLASS (manager);
 
@@ -1313,8 +1364,11 @@ gf_monitor_manager_handle_change_backlight (GfDBusDisplayConfig   *skeleton,
       return TRUE;
     }
 
-  if (output->backlight == -1 ||
-      (output->backlight_min == 0 && output->backlight_max == 0))
+  output_info = gf_output_get_info (output);
+
+  if (gf_output_get_backlight (output) == -1 ||
+      (output_info->backlight_min == 0 &&
+       output_info->backlight_max == 0))
     {
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
                                              G_DBUS_ERROR_INVALID_ARGS,
@@ -1324,8 +1378,10 @@ gf_monitor_manager_handle_change_backlight (GfDBusDisplayConfig   *skeleton,
 
   manager_class->change_backlight (manager, output, value);
 
-  gf_dbus_display_config_complete_change_backlight (skeleton, invocation,
-                                                    output->backlight);
+  new_backlight = gf_output_get_backlight (output);
+  gf_dbus_display_config_complete_change_backlight (skeleton,
+                                                    invocation,
+                                                    new_backlight);
 
   return TRUE;
 }
@@ -1650,12 +1706,6 @@ gf_monitor_manager_handle_get_current_state (GfDBusDisplayConfig   *skeleton,
     }
 
   capabilities = gf_monitor_manager_get_capabilities (manager);
-  if ((capabilities & GF_MONITOR_MANAGER_CAPABILITY_MIRRORING) == 0)
-    {
-      g_variant_builder_add (&properties_builder, "{sv}",
-                             "supports-mirroring",
-                             g_variant_new_boolean (FALSE));
-    }
 
   g_variant_builder_add (&properties_builder, "{sv}",
                          "layout-mode",
@@ -1941,11 +1991,14 @@ gf_monitor_manager_real_is_lid_closed (GfMonitorManager *manager)
 static void
 gf_monitor_manager_real_read_current_state (GfMonitorManager *manager)
 {
+  GfMonitorManagerPrivate *priv;
   GList *l;
 
+  priv = gf_monitor_manager_get_instance_private (manager);
+
   manager->serial++;
 
-  for (l = manager->gpus; l; l = l->next)
+  for (l = gf_backend_get_gpus (priv->backend); l; l = l->next)
     {
       GfGpu *gpu = l->data;
       GError *error = NULL;
@@ -2060,7 +2113,6 @@ gf_monitor_manager_finalize (GObject *object)
 
   manager = GF_MONITOR_MANAGER (object);
 
-  g_list_free_full (manager->gpus, g_object_unref);
   g_list_free_full (manager->logical_monitors, g_object_unref);
 
   G_OBJECT_CLASS (gf_monitor_manager_parent_class)->finalize (object);
@@ -2281,25 +2333,18 @@ gf_monitor_manager_get_primary_logical_monitor (GfMonitorManager *manager)
   return manager->primary_logical_monitor;
 }
 
-void
-gf_monitor_manager_add_gpu (GfMonitorManager *manager,
-                            GfGpu            *gpu)
-{
-  manager->gpus = g_list_append (manager->gpus, gpu);
-}
-
-GList *
-gf_monitor_manager_get_gpus (GfMonitorManager *manager)
-{
-  return manager->gpus;
-}
-
 gboolean
 gf_monitor_manager_has_hotplug_mode_update (GfMonitorManager *manager)
 {
+  GfMonitorManagerPrivate *priv;
+  GList *gpus;
   GList *l;
 
-  for (l = manager->gpus; l; l = l->next)
+  priv = gf_monitor_manager_get_instance_private (manager);
+
+  gpus = gf_backend_get_gpus (priv->backend);
+
+  for (l = gpus; l; l = l->next)
     {
       GfGpu *gpu = l->data;
 
